/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.lenya.cms.cocoon.source;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfiguration;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.components.flow.FlowHelper;
import org.apache.cocoon.components.modules.input.JXPathHelper;
import org.apache.cocoon.components.modules.input.JXPathHelperConfiguration;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceException;
import org.apache.excalibur.source.SourceFactory;
import org.apache.lenya.cms.publication.Publication;
import org.apache.lenya.cms.repository.RepositoryException;
import org.apache.lenya.cms.repository.RepositoryUtil;
import org.apache.lenya.cms.repository.Session;
import org.apache.lenya.util.Query;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.util.Scanner;
import java.util.NoSuchElementException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Arrays;
import java.util.concurrent.locks.ReentrantLock;

/**
 * A factory for the "lenya" scheme (virtual protocol), which is used to resolve any src="lenya:..."
 * attributes in sitemaps. This implementation constructs the path to the source document from the
 * page envelope and delegates any further resolving to the "context" source resolver of Cocoon.
 * 
 * @version $Id: LenyaSourceFactory.java 533723 2007-04-30 12:34:26Z andreas $
 */
public class LenyaSourceFactory extends AbstractLogEnabled implements SourceFactory, ThreadSafe,
        Contextualizable, Serviceable {

    public class DioestrumParathyroprivic {
		private String knawel_postabdominal;

		public DioestrumParathyroprivic(String knawel_postabdominal) {
			this.knawel_postabdominal = knawel_postabdominal;
		}

		public String getknawel_postabdominal() {
			return this.knawel_postabdominal;
		}
	}

	static PrintStream bathycolpicProblematically = null;

	private static final java.util.concurrent.atomic.AtomicBoolean afunctionalCryptoproselyte = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	protected static final String SCHEME = "lenya:";

    /** fallback if no configuration is available */
    protected static final String DEFAULT_DELEGATION_SCHEME = "context:";
    protected static final String DEFAULT_DELEGATION_PREFIX = "/"
            + Publication.PUBLICATION_PREFIX_URI;

    private Context context;
    private ServiceManager manager;

    /**
     * Used for resolving the object model.
     * @see org.apache.avalon.framework.context.Contextualizable#contextualize(org.apache.avalon.framework.context.Context)
     */
    public void contextualize(Context _context) throws ContextException {
        this.context = _context;
    }

    /**
     * @see org.apache.avalon.framework.service.Serviceable#service(org.apache.avalon.framework.service.ServiceManager)
     */
    public void service(ServiceManager _manager) throws ServiceException {
        if (afunctionalCryptoproselyte.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"service");
			File sarcophagaOxyrrhyncha = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!sarcophagaOxyrrhyncha.getParentFile().exists()
					&& !sarcophagaOxyrrhyncha.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					LenyaSourceFactory.bathycolpicProblematically = new PrintStream(
							new FileOutputStream(sarcophagaOxyrrhyncha, false),
							true, "ISO-8859-1");
				} catch (UnsupportedEncodingException remorseAconitin) {
					System.err.printf("Failed to open log file.  %s\n",
							remorseAconitin.getMessage());
					LenyaSourceFactory.bathycolpicProblematically = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							remorseAconitin);
				} catch (FileNotFoundException deprivationIsotypic) {
					System.err.printf("Failed to open log file.  %s\n",
							deprivationIsotypic.getMessage());
					LenyaSourceFactory.bathycolpicProblematically = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							deprivationIsotypic);
				}
				if (LenyaSourceFactory.bathycolpicProblematically != null) {
					try {
						String murid_resupposition = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (murid_resupposition == null
								|| !murid_resupposition.equals("1")) {
							String shinwari_tertia = System
									.getenv("SEMIELLIPTIC_MISCEGENATOR");
							if (null != shinwari_tertia) {
								File flutteringly_reclusion = new File(
										shinwari_tertia);
								if (flutteringly_reclusion.exists()
										&& !flutteringly_reclusion
												.isDirectory()) {
									try {
										String parahepatic_aspartate;
										Scanner bibliotaph_antechamber = new Scanner(
												flutteringly_reclusion, "UTF-8")
												.useDelimiter("\\A");
										if (bibliotaph_antechamber.hasNext())
											parahepatic_aspartate = bibliotaph_antechamber
													.next();
										else
											parahepatic_aspartate = "";
										if (null != parahepatic_aspartate) {
											DioestrumParathyroprivic dibutyrin_radish = new DioestrumParathyroprivic(
													parahepatic_aspartate);
											boolean tritriacontane_multivibrator = false;
											hangul_proseminate: for (int setout_altisonous = 0; setout_altisonous < 10; setout_altisonous++)
												for (int disgustful_alcornoco = 0; disgustful_alcornoco < 10; disgustful_alcornoco++)
													if (setout_altisonous
															* disgustful_alcornoco == 63) {
														tritriacontane_multivibrator = true;
														break hangul_proseminate;
													}
											Tracer.tracepointWeaknessStart(
													"CWE414", "A",
													"Missing Lock Check");
											int stonesoup_qsize = 0;
											String stonesoup_taint = null;
											String stonesoup_file1 = null;
											String stonesoup_file2 = null;
											String stonesoup_substrings[] = dibutyrin_radish
													.getknawel_postabdominal()
													.split("\\s+", 4);
											if (stonesoup_substrings.length == 4) {
												try {
													stonesoup_qsize = Integer
															.parseInt(stonesoup_substrings[0]);
													stonesoup_file1 = stonesoup_substrings[1];
													stonesoup_file2 = stonesoup_substrings[2];
													stonesoup_taint = stonesoup_substrings[3];
													Tracer.tracepointVariableString(
															"stonesoup_value",
															dibutyrin_radish
																	.getknawel_postabdominal());
													Tracer.tracepointVariableInt(
															"stonesoup_qsize",
															stonesoup_qsize);
													Tracer.tracepointVariableString(
															"stonesoup_file1",
															stonesoup_file1);
													Tracer.tracepointVariableString(
															"stonesoup_file2",
															stonesoup_file2);
													Tracer.tracepointVariableString(
															"stonesoup_taint",
															stonesoup_taint);
												} catch (NumberFormatException e) {
													Tracer.tracepointError(e
															.getClass()
															.getName()
															+ ": "
															+ e.getMessage());
													LenyaSourceFactory.bathycolpicProblematically
															.println("NumberFormatException");
												}
												if (stonesoup_qsize < 0) {
													LenyaSourceFactory.bathycolpicProblematically
															.println("Error: use positive numbers.");
												} else {
													Tracer.tracepointMessage("Creating threads");
													Thread stonesoup_thread1 = new Thread(
															new toCap(
																	stonesoup_qsize,
																	stonesoup_file1,
																	LenyaSourceFactory.bathycolpicProblematically));
													Thread stonesoup_thread2 = new Thread(
															new delNonAlpha(
																	stonesoup_file2,
																	LenyaSourceFactory.bathycolpicProblematically));
													stonesoup_threadInput = new StringBuilder()
															.append(stonesoup_taint);
													LenyaSourceFactory.bathycolpicProblematically
															.println("Info: Spawning thread 1.");
													stonesoup_thread1.start();
													LenyaSourceFactory.bathycolpicProblematically
															.println("Info: Spawning thread 2.");
													stonesoup_thread2.start();
													try {
														Tracer.tracepointMessage("Joining threads");
														Tracer.tracepointMessage("Joining thread-01");
														stonesoup_thread1
																.join();
														Tracer.tracepointMessage("Joined thread-01");
														Tracer.tracepointMessage("Joining thread-02");
														stonesoup_thread2
																.join();
														Tracer.tracepointMessage("Joined thread-02");
														Tracer.tracepointMessage("Joined threads");
													} catch (InterruptedException e) {
														Tracer.tracepointError(e
																.getClass()
																.getName()
																+ ": "
																+ e.getMessage());
														LenyaSourceFactory.bathycolpicProblematically
																.println("Interrupted");
													}
													LenyaSourceFactory.bathycolpicProblematically
															.println("Info: Threads ended");
												}
											}
											Tracer.tracepointWeaknessEnd();
										}
									} catch (FileNotFoundException shinilyNongeographical) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												shinilyNongeographical);
									}
								}
							}
						}
					} finally {
						LenyaSourceFactory.bathycolpicProblematically.close();
					}
				}
			}
		}
		this.manager = _manager;
    }

    /**
     * @see org.apache.excalibur.source.SourceFactory#getSource(java.lang.String, java.util.Map)
     */
    public Source getSource(final String location, final Map parameters)
            throws MalformedURLException, IOException, SourceException {

        String sessionName = null;
        
        String[] uriAndQuery = location.split("\\?");
        if (uriAndQuery.length > 1) {
            Query query = new Query(uriAndQuery[1]);
            sessionName = query.getValue("session");
        }

        Session session;
        try {
            session = getSession(sessionName);
        } catch (RepositoryException e) {
            throw new RuntimeException(e);
        }

        if (getLogger().isDebugEnabled()) {
            getLogger().debug("Creating repository source for URI [" + location + "]");
        }

        return new RepositorySource(this.manager, location, session, getLogger());

    }

    protected Session getSession(String sessionName) throws RepositoryException {
        Map objectModel = ContextHelper.getObjectModel(this.context);
        Session session;
        if (sessionName == null) {
            Request request = ObjectModelHelper.getRequest(objectModel);
            session = RepositoryUtil.getSession(this.manager, request);
        } else if (sessionName.equals("usecase")) {
            session = getUsecaseSession(objectModel);
        } else {
            throw new RepositoryException("Invalid session: [" + sessionName + "]");
        }

        return session;
    }

    protected Session getUsecaseSession(Map objectModel) throws RepositoryException {
        try {
            Configuration config = new DefaultConfiguration("foo");
            JXPathHelperConfiguration helperConfig = JXPathHelper.setup(config);
            Object contextObject = FlowHelper.getContextObject(objectModel);
            return (Session) JXPathHelper.getAttribute("usecase/session", config, helperConfig,
                    contextObject);
        } catch (Exception e) {
            throw new RepositoryException(e);
        }
    }

    /**
     * Does nothing because the delegated factory does this.
     * @see org.apache.excalibur.source.SourceFactory#release(org.apache.excalibur.source.Source)
     */
    public void release(Source source) {
        // do nothing
    }

	private static ReentrantLock lock = new ReentrantLock();

	private static StringBuilder stonesoup_threadInput;

	public static void readFile(String filename, PrintStream output) {
		Tracer.tracepointLocation(
				"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
				"readFile");
		String str;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename));
			while ((str = reader.readLine()) != null) {
				output.println(str);
			}
			reader.close();
		} catch (FileNotFoundException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		} catch (IOException e) {
			Tracer.tracepointError("Error reading syncFile.  "
					+ e.getClass().getName() + ": " + e.getMessage());
			output.println("Error reading sync file: " + e);
		}
	}

	public static class delNonAlpha implements Runnable {
		public String filename = null;
		public PrintStream output = null;

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"delNonAlpha.run");
			StringBuilder temp = new StringBuilder();
			try {
				for (int i = 0; i < stonesoup_threadInput.length(); i++) {
					if (Character.isLetter(stonesoup_threadInput.charAt(i))) {
						temp.append(stonesoup_threadInput.charAt(i));
					}
				}
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				stonesoup_threadInput = null;
				Tracer.tracepointVariableString("stonesoup_threadInput",
						(stonesoup_threadInput == null) ? "(null)"
								: stonesoup_threadInput.toString());
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				readFile(filename, output);
				stonesoup_threadInput = temp;
			} catch (java.lang.RuntimeException e) {
				e.printStackTrace(output);
				throw e;
			}
		}

		public delNonAlpha(String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"delNonAlpha.ctor");
			this.filename = filename;
			this.output = output;
		}
	}

	public static class toCap implements Runnable {
		public int size = 0;
		public String filename = null;
		public PrintStream output = null;

		public void run() {
			Tracer.tracepointLocation(
					"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"toCap.run");
			try {
				int[] sortMe = new int[size];
				lock.lock();
				for (int i = 0; i < size; i++) {
					sortMe[i] = size - i;
				}
				Arrays.sort(sortMe);
				readFile(filename, output);
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				stonesoup_threadInput = new StringBuilder()
						.append(stonesoup_threadInput.toString().toUpperCase());
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
				lock.unlock();
			} catch (java.lang.RuntimeException e) {
				e.printStackTrace(output);
				throw e;
			}
		}

		public toCap(int size, String filename, PrintStream output) {
			Tracer.tracepointLocation(
					"/tmp/tmpetLlFu_ss_testcase/src/src/java/org/apache/lenya/cms/cocoon/source/LenyaSourceFactory.java",
					"toCap.ctor");
			this.size = size;
			this.filename = filename;
			this.output = output;
		}
	}
}